import sys
import os
import threading
import tkinter as tk
from tkinter import ttk
import RPi.GPIO as GPIO
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import time
# Add the parent directory to the module search path
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))
from interface_board_pins import *  # Import pin assignments


# CONFIG
DIGITAL_PLOT_UPDATE_DELAY_MS = 50  # note: delay is additional to time it takes to update the chart
TAB_NOT_ACTIVE_CHECK_INTERVAL_MS = 2000   # Check inactive tabs less frequently
MAX_X_HISTORY_VALUES = 100         # Keep last 100 values for scrolling
DEBUG = False


# Variables
updating_enabled = False  # Track if the tab is active
input_channels = list(range(8))
data = {ch: [0] * MAX_X_HISTORY_VALUES for ch in input_channels}  # Preallocate data
time_data = list(range(-MAX_X_HISTORY_VALUES, 0))  # Simulated time axis


def create_digital_plot_tab(notebook):
    frame = ttk.Frame(notebook)
    notebook.add(frame, text="Digital Inputs")

    figure = Figure(figsize=(8, 5), dpi=100)
    ax = figure.add_subplot(1, 1, 1)

    ax.set_title("Digital Inputs")  # Set title
    ax.set_xlabel("Time (s)")  # X-axis label
    ax.set_ylabel("State (1=HIGH / 0=LOW)")  # Y-axis label

    ax.set_ylim(-0.2, 1.2)
    ax.set_xlim(-MAX_X_HISTORY_VALUES, 0)  # Keep time axis fixed
    canvas = FigureCanvasTkAgg(figure, master=frame)
    canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    # Initialize lines for fast updates
    lines = {}
    for ch in input_channels:
        label = f"Digital In {ch}"
        lines[ch] = ax.step(time_data, data[ch], where="post", label=label)[0]

    ax.legend()  # Show legend

    def update_plot():
        if not updating_enabled:
            frame.after(TAB_NOT_ACTIVE_CHECK_INTERVAL_MS, update_plot)
            return
        
        # Shift existing data left
        for ch in input_channels:
            data[ch].pop(0)
            data[ch].append(GPIO.input(GPIO_DIGITAL_INPUTS[ch]))

        # Update only the y-data for efficiency
        for ch in input_channels:
            lines[ch].set_ydata(data[ch])

        canvas.draw_idle()  # More efficient than draw()
        frame.after(DIGITAL_PLOT_UPDATE_DELAY_MS, update_plot)

    update_plot()



def set_updating_enabled(is_active):
    global updating_enabled
    updating_enabled = is_active
    if DEBUG: print(f"digital_plot tab: set updating_enabled to {updating_enabled}")

